
;*******************************************************************************
; 
; Program:	Gorf Memory Diagnostic Program
;
; File Name:	GORFMD.ASM
;
; Compiler:	ZMAC Assembler
;		To compile use "zmac.exe -c -m GORFMD.ASM"
;
; Author:	David E. Turner cmdr-dave@spamcop.net
;		
; Copyright:	Copyright 2005 by author. 
;		
;		This code and source file is free to use by individuals
;		in a non-commercial manner. It may be freely distributed
;		unaltered. Although distribution is free, the author retains
;		all rights.
;
;		This code or binary file may not be used for any commercial 
;		purpose including, but not limited to, distribution that requires
;		a purchase beyond the cost of media and/or actual shipping cost,
;		sale on eBay or other commercial site, publication for profit,
;		or inclusion in other works that is for sale.	
;
; Version:	1.0 - First release to public. 
;		This code has been cleaned up a bit for public distribution,
;		however there are many comment errors and also much better 
;		ways of doing things. Before passing judgement, please remember
;		that I am not a professional programmer... it's just a hobby.
;  
; Purpose:	Finds bad video memory on a Gorf arcade game and reports it's location. 
;		Does not use any pattern board functions, so pattern board can be
;		removed if so desired. This may be useful because a pattern board
;		failure sometimes looks like bad video memory.
;		
; Limitations:	Many... memory test is far from extensive and assumes all other
;		hardware is working correctly. It also will report bad memory
;		if there are other problems on the RAM board such as bad driver
;		logic chips. Also, this program has not been fully debugged.
;
;		THIS PROGRAM IS FOR USE AS-IS. THE AUTHOR ASSUMES NO RESPONSIBILTY
;		OF IT'S FITTNESS FOR USE AND CANNOT BE HELD RESPONSIBLE FOR ANY
;		DAMAGES OR LOSS WHETHER DIRECT OR INDIRECT. THE USER OF THIS PROGRAM
;		ASSUMES ALL RESONSIBILITY AND USES IT AT HIS OR HER OWN RISK.
;
;*******************************************************************************
; 
; Equate table:
; 
;*******************************************************************************
;

VIDMEM		EQU	$4000			; Beginning of video memory
VMSIZE		EQU	$3FFF			; Size of video memory including hidden area
VMSIZEV		EQU	VMSIZE - $40		; Viewable video memory only
STACK		EQU	$D400			; Top of Stack space
ASCIIDGT	EQU	$D303			; 3 locations to hold ASCII to be printed
SPOS		EQU	$D300			; Last cursor position

;
;
;*******************************************************************************
; 
; Begin main code
; 
;*******************************************************************************
;
	ORG	$0000				; Beginning of ROM (see memory map)                           

	di					; No Interruptions
	ld	sp,STACK			; Set stack

	call	INITHW				; Initilize the hardware (video & other)
	call	CLRSCRN				; Clear the screen

	
	ld	a,00001100b			; Red Letters
	ld	de,560*1-(4*1)-480		; Line 1, Char 1 
	call	PUTSTRB				; Write the string
	db	"GORF VIDEO MEMORY TEST",$ff

	ld	a,00001100b
	ld	de,560*1-(4*2)-480		; Line 2, Char 1 
	call	PUTSTRB				; Write the string
	db	"PRESS PLAYER 1 BUTTON",$ff

	ld	a,00001100b
	ld	de,560*1-(4*3)-480		; Line 3, Char 1 
	call	PUTSTRB				; Write the string
	db	"TO START",$ff	

	ld	a,00001100b
	ld	de,560*1-(4*5)-480		; Line 5, Char 1 
	call	PUTSTRB				; Write the string
	db	"[2005 DAVID E. TURNER",$ff	

	ld	a,00001100b
	ld	de,560*1-(4*6)-480		; Line 6, Char 1 
	call	PUTSTRB				; Write the string
	db	"CMDR-DAVE@SPAMCOP.NET",$ff

	ld	a,00001100b
	ld	de,560*1-(4*7)-480		; Line 7, Char 1 
	call	PUTSTRB				; Write the string
	db	"VERSION 1.0",$ff

	ld	a,00001100b
	ld	de,560*1-(4*10)-480		; Line 10, Char 1 
	call	PUTSTRB				; Write the string
	db	"NOT FOR RESALE",$ff

	ld	a,00001100b			; Red Letters
	ld	de,560*1-(4*11)-480		; Line 11, Char 1 
	call	PUTSTRB				; Write the string
	db	"FREE DISTRIBUTION ONLY",$ff


	ld	a,00001100b			; Red Letters
	ld	de,560*1-(4*12)-480		; Line 12, Char 1 
	call	PUTSTRB				; Write the string
	db	"SUGGESTIONS AND BUG",$ff

	ld	a,00001100b			; Red Letters
	ld	de,560*1-(4*13)-480		; Line 13, Char 1 
	call	PUTSTRB				; Write the string
	db	"REPORTS WELCOME",$ff


START_LOOP:
	
	in	a,($10)				; Get P1 START Button Status (resets Watchdog too)
	bit	4,a				; Bit 4 active low = button pressed
	jp	nz,START_LOOP			; Button not pressed, loop

RESTART_TEST:					; This label is for restarting

	call	CLRSCRN				; Clear the screen
	call	SMEMT				; Short static memory test 
	call	VMEMT				; Short video memory test 


	ld	a,00001100b			; Red Letters
	ld	de,560*1-(4*8)-480		; Line 8, Char 1
	call	PUTSTRB				; Write the string
	db	"END MEMORY TESTS",$ff

	ld	a,00001100b			; Red Letters
	ld	de,560*1-(4*10)-480		; Line 10, Char 1 
	call	PUTSTRB				; Write the string
	db	"PRESS PLAYER 1 BUTTON",$ff

	ld	a,00001100b			; Red Letters
	ld	de,560*1-(4*11)-480		; Line 11, Char 1 
	call	PUTSTRB				; Write the string
	db	"TO RESTART TEST",$ff

ELOOP:	

	in	a,($10)				; Get P1 START Button Status (resets Watchdog too)
	bit	4,a				; Bit 4 active low = button pressed
	jp	nz,ELOOP			; Button not pressed, loop

	jp	RESTART_TEST
;
;******************************************************************************
;Subroutines start here:
;******************************************************************************
;
;******************************************************************************
;
; Setup for short STATIC memory test...
;
;******************************************************************************
;
; =====> PROGRAM STUB!!! Future expansion... <========

SMEMT:	ret		; Return to sender...

;
;
;******************************************************************************
;
; Name: VMEMT
;
; This is the simplifed memory test for video memory. It is a simple zero
; fill with a check after every byte written. I have to do the check that often 
; because if there is an error, HL has to contain the memory address so I can
; find which chip is bad. 
;
; Registers destroyed: AF, BC, DE, HL, AF'
;
; Entry Parameters:
; A= Pattern for test - $00 normally 
; HL= First memory address to be tested
;
; Normal Exit Parameters:
; None - Nothing important to return. No news is good news.
; 
; Exception Exit parameters: 
; A  = Test Pattern where error occurred (tells which bit was bad)
; HL = Location in memory where error occurred.
; 
;******************************************************************************
;
VMEMT:		
		ld	a,11111111b		; All set Pattern
		ld	hl,VIDMEM		; Point to beginning of video memory  
		ld	bc,VMSIZE		; Video memory size
		call	MFILL			; Fill video memory with zero's

		call	KICKDOG			; Reset Watchdog

		ld	a,11111111b		; pattern to check
		ld	hl,VIDMEM		; Reload base of video memory
		ld	bc,VMSIZE		; Video memory size
		call	MCOMP			; Compare memory against value
		cpl				; put error bits as set, non-error as reset

		call	KICKDOG			; Reset Watchdog

		jp	nz,VMEMTE		; jump to error routine if bad memory detected
						; this is a jump to simplify error message handling

		ld	a,00000000b		; Zero Pattern - Want to detect stuck bits
		ld	hl,VIDMEM		; Point to beginning of video memory  
		ld	bc,VMSIZE		; Video memory size
		call	MFILL			; Fill video memory with zero's

		call	KICKDOG			; Reset Watchdog

		ld	a,00000000b		; pattern to check
		ld	hl,VIDMEM		; Reload base of video memory
		ld	bc,VMSIZE		; Video memory size
		call	MCOMP			; Compare memory against value

		call	KICKDOG			; Reset Watchdog

		jp	nz,VMEMTE		; jump to error routine if bad memory detected
						; this is a jump to simplify error message handling

		; Report that memory passed the test

		ld	a,00001100b		; Red Letters
		ld	de,560*1-(4*3)-480	; Line 3, Char 1
		push	hl
		call	PUTSTRB		
		db	"ALL VIDEO MEMORY TESTS GOOD",$ff
		pop	hl
		
		ret			; Return if all memory is good

;
;******************************************************************************
;
; Name: VMEMTE
;
; This is called from the memory fill routine when it gets an error 
; (See VMEMT:). This tries to find the bad chip based on memory location and 
; the bit. This is limited to the first bad chip found for now.
;
; Registers changed: AF, BC (Registers are not saved)
;
; Entry Parameters:
;   A  =  Pattern that failed (which bit)
;   HL = Location in memory where it failed
;
; Normal Exit Parameters:
;   Returns to the program that called VMEMT. Later it will set the
;   carry flag (or other indicator) to tell the main program of error.	
; 
; Exception Exit parameters: 
;   No exceptions. It just calculates the bad chip and writes an error
;   message to the screen.	
;
;******************************************************************************
;
		
VMEMTE:		call	CLRSCRN			; Clear the screen

		push	af
		ld	a,00001100b		; RED letters
		ld	de,560*1-(4*3)-480	; Line 3, Char 1 
		push	hl
		call	PUTSTRB		
		db	"ERROR ON CARD ",$ff
		pop	hl
		pop	af

		; Note: A holds the pattern in error

		; Figure out which card, slot 5 or slot 6?

		push	hl
		push	af			; Save the pattern for other testing

		and	10101010b		; Mask to have only column 6 bits only
		ld	a,6			; Set up to write a 6 
		jr	z,CARD6			; Z - card 6, NZ - card 5
		dec	a			; Must be card 5.

CARD6:		ld	de,(SPOS)		; go print the number at last cursor location
		call	PUTSTRN			;

		pop	af			; Get back the failing pattern for column test

		; Column test
		
		ld	b,$03			; Set initial bit number
SHFTLP:		rla				; Rotate into carry
		jr	c,FNDIT			; Carry is set? We have a winner!
		rla				;
		jr	c,FNDIT
		djnz	SHFTLP			; Carry wasn't set, go try again

		
FNDIT:		
		ld	a,b			; Get row number [0-3], but reverse order
		cpl				; Complement A to get it in correct order
		and	00000011b		; Mask off all but numbers 0-3
		ld	b,a			; Reg B now has which column [0-3]

	
; This part finds what row the bad chip is located using the address in HL.
; It does this by masking off all but bits [1:0] in register L. 
; These bits make up the ROW number [0:3]


		pop	hl
		ld	a,l			; Let's determine which row bad chip is located
		and	00000011b		; mask off all but last two bits
						; Reg A now has row, 0-3 in reverse order
		

		
; Now the tricky part... change the row and column numbers into a Uxx number. 
; Chips on the board are numbered U1 through U16, from RIGHT to LEFT on a 4x4 grid.
; At this point, B = Column and A = Row (in reverse order - remember, numbering is 
; backwards on the card.

; Here is the formula for changing the ROW and COLUMN (reversed) into the correct Uxx number
; on the RAM board. I don't know WHY it works, I just stumbled upon it. May my lack of 
; understanding be compensated by my accuracy in duplication... :-)
; Formula is: U<xx>=4*R+C+1 

		
		add	a,a			; Multiply the row *4
		add	a,a			; 
		add	a,b			; Subtract off the column
		inc	a			; Register A now has the U number 1-16
	

		push	af
		ld	a,00001100b		; Red Letters
		ld	de,560*1-(4*4)-480	;Line 4, Char 1
		call	PUTSTRB			; Write the string below
		db	"CHIP NUMBER U",$ff
		pop	af

		ld	de,(SPOS)
		call	PUTSTRN			; Writes out a numeric in the A register.

		ret				; End of memory test
						
;
;
;******************************************************************************
;				
; Begin video & other necessary initilization
;
;******************************************************************************
;
INITHW:	ld	a,$01			; Set Video to High-Res                            
	out	($08),a	 

	ld	a,$2C			; Set video color and mode  
	out	($09),a			; Bit 0-5 = set right/left palette 
					; Bit 6-7 = background color according to pallette 
				
	ld	a,$CC			; =204                                                        
	out	($0A),a			; Set screen height                                           

	ld	a,$CA			;Interrupt vector out to $CA (202)         
	out	($0D),a			;Set interrupt vector  

	ld	hl,COLTAB		; Point to color table             
	ld	b,$08			; 8 bytes to output
	ld	c,$0b			; Color table Output port 
	otir				;(C)<-(HL), HL+1, B-1 until B=0            
				          
	ld	a,00000000b		; no screen, no light pen Interrupts                                                      
	out	($0E),a			;
	
	ld	 a,202			; Screen interrupt line.
	out	 ($0F),a		; Even though we are not using it, init it anyway.

	ld	 a,$00			; This routine was taken from Wizard of Wor. 
	ld	 i,a			; Set High order byte for Interrupt                          
	im	 2			; A call is made to an address read from address 
					; (register I  256 + value from interrupting device)        
					; Looks like interrupts will be from $0000 to $00FF
	
	ret				; ... and we're outta here!

;
;******************************************************************************
;
; Name: KICKDOG
;
; Purpose: Resets the watchdog timer
;
;******************************************************************************
; 

KICKDOG:	
	ex	af,af'
	in	a,($10)			; Kick the dog...
	ex	af,af'



;******************************************************************************
;
; Name: PUTSTRN
;
; Description: This sets up to write the number in register A to the screen.
;	       
; Registers used: A, B, C, HL, 
;
; Entry Parameters:
;   A = Number to be translated into characters and written to screen
;   DE = Location on screen (Magic RAM). 
;
; Normal Exit Parameters:
;   Registers are set suitable for the PUTSTR routine and jumps to it. 
;   See PUTSTR for the parameters that it expects. 
; 
; Exception Exit parameters: 
;   No exceptions at this time. PUTSTR will (eventually) handle the error
;   conditions. Another that may be put in is if character string is too long
;	
;******************************************************************************
;

PUTSTRN:	
		push	hl
		push	af
		push	bc

		ld	hl, ASCIIDGT+2	; Point to third save location
		call	GASCIID		; Get ASCII Digit
		ld	(hl),a		; save the digit
		ld	a,b		; Get last quotient in A again
		dec	hl		; Point to second save location
		call	GASCIID		; Get ASCII Digit
		ld	(hl),a		; save the digit
		ld	a,b		; Get last quotient in A again
		dec	hl		; Point to first save location
		call	GASCIID		; Get ASCII digit
		ld	(hl),a		; Save the digit
		
		
					; A still has the first digit
		ld	b,3		; Set character count for all three
		cp	'0'		; Is the first digit a zero?
		jr	nz,PSNSKIP1	; Good leading number, skip mods
		inc	hl		; Make starting point at second char
		dec	b		; Set to two digits
		ld	a,(hl)		; Get the second character
		cp	'0'		; Is the second digit a zero also?
		jr	nz,PSNSKIP1	; Skip the mods
		inc	hl		; point to third digit
		dec	b		; Set for only one digit to write


PSNSKIP1:	ld	a,00001100b	; RED Numbers
		call	PUTSTR		; Parameters are set, so write it!
		
		pop	bc
		pop	af
		pop	hl
		ret			; return to sender

;
;******************************************************************************
;
; Name: GASCIID
;
; Description: This translates the binary number in A into it's last
;	       last ASCII digit. It is meant to be called repeatedly with 
;	       to get all the ASCII digits in a binary number. It gets the
;	       last ASCII digit by dividing by 10 and converting the remainder
;	       into the ASCII digit.
;
; Registers used: ???
;
; Entry Parameters:
;
;   
;
; Normal Exit Parameters:
;	  
;
;  
; 
; Exception Exit parameters: 
;   None. Currently no error checking.
;	
;******************************************************************************
;

GASCIID:	
		
		
		ld	b,-1		; 
GASCIID1	inc	b
		sub	10		; Subtract 10 until...
		jr	nc,GASCIID1	;   ...it goes negative
		add	a,10		; bump it back up one to get remainder
		add	a,$30		; change to ASCII digit
					; Remainder is in A
					; Quotient in B	
		

		ret

;
;******************************************************************************
;
; Name: PUTSTRB
;
; Description: Takes the data string below the call statement, puts it in a form
;  so that the PUTSTR can write it, calls PUTSTR, sets correct return addres and 
;  returns. The terminator is $FF.
;
; Registers used: ???
;
; Entry Parameters:
;   None. It takes data after call instruction so nothing is passed.
;
; Normal Exit Parameters:
;   None. String is written by PUTSTR and this returns to next valid 
;   instruction. See PUTSTR for the parameters that it expects. 
; 
; Exception Exit parameters: 
;   None. PUTSTR will (eventually) handle the error
;   conditions. 
;	
;******************************************************************************
;

PUTSTRB:	pop	hl		; pull the return address off of the 
					; stack. That is where the data starts.
		
		push	af		; Save the "Expand mode color"
		push	de		; Save the screen position

		push	hl		; Start figuring length
		pop	de

		ld	b,-1		; Set initial string length to one
PUTSTRBLP1:	inc	b		; Increment count in case of loop
		ld	a,(de)		
		cp	($ff)		; Is character a terminator?
		inc	de		; Point to next character in case of loop
		jr	nz,PUTSTRBLP1	; Loop until EOL
		
		pop	de		; Restore screen position
		pop	af		; Restore "Expand Mode Color" 

; So at this point, we have met all requirements for calling PUTSTR.
		
		call	PUTSTR		; Write string to screen
					; ??? assumes HL is saved in PUTSTR
		ld	c,b		
		ld	b,0
		add	hl,bc		; Add the length of the string 
		inc	hl		; and one for the terminator
		push	hl		; put it as the return address
		ret			; And return to code after the data string.

;
;******************************************************************************
;
; Name: PUTSTR
;
; Description: Writes an ASCII string to screen. 
;
; Registers changed: 
;   None. (if register not listed, it is saved)
;   
;
; Entry Parameters:
;   A =  Expand mode color (color of string)
;   B =  Length of string (ASCII Character count)
;	 Length cannot be zero. If call with a zero,
;	 it will write 256 characters.
;   DE = Position in "magic RAM"
;   HL = Start address of string (ASCII)
;   
; Normal Exit Parameters:
;   None ???
; 
; Exception Exit parameters: 
;
; Special Note:
;	The formula for printing to screen in expanded mode is:
;	DE = 76 - 4*(Line Number-1) + 560*(character position - 1)
;	OR
;	DE = 560 * (Character Position) - 4 * (Line Number) - 480
;	where "line number" is 1-20 and "character position" is 1-29.
;	If you are using this formula at compile time, note that 
;	some compilers do not use operator precidence and you
;	have to add extra parantheses to get it to calculate correctly.
;	
;	
;******************************************************************************

PUTSTR:	
	push	af		; Save registers... 
	push	bc
	push	de
	push	hl

	out	($19),a		; Set the expand register (color)
	
;	ld	a,c		; Set Magic Register.. (later expansion)	
	ld	a,$08		; Set expand mode ONLY
	out	($0c),a

PUTSTR1:
	ld	a,(hl)		; Get the character of the string
	call	PUTCHAR		; Go print the character
	inc	hl		; Bump the character pointer
	djnz	PUTSTR1		; go back for more...
			
	ld	(SPOS),de	; Save the last cursor location
	pop	hl		; Save registers... may be temporary.
	pop	de
	pop	bc
	pop	af

	ret			; String is done, return to sender...

;
;******************************************************************************
;
; Name: PUTCHAR
;
; Puts one ASCII character to screen starting at an absolute address.
; In other words, the character can start at any video memory address.
;
; Notes: This subroutine can be written much better, but at the time
;	 of this writing, I am in a big hurry to finish. I'll optimize
;	 later.
;   
; Entry Parameters:
;	   
;	Both the "Expand Mode Color" register ($0C) and the "Magic
;	Register" ($0B) must already be set when entering this 
;	routine. 
;
;	A  = ASCII character to be printed (see valid characters below)
;
;	DE = Destination in "Magic RAM". See writeup on Magic RAM for 
;	     details. In Gorf, it is $0000-$3FFF.
;
; Registers destroyed:
;	B  = Used for counter
;	HL = Used to point to character bitmap table
;   
;   
; Normal Exit Parameters:
;	DE = Next character position (frame) 
; 
; Exception Exit parameters: 
;	Routine does nothing if outside range of printable characters
;	Certain characters will print a space until bitmap updated. 
;	These are the ! " # $ % & ' ( ) * + , - . / ; < = > ? @ characters.
;	
;
; Subroutines Used:
;	 ASCII_LOOKUP	Takes ASCII char in A and converts it
;			to the starting address for the bitmap
;			which is returned in HL.
;			 
;	
;********************************************************************

PUTCHAR:
	push	hl		; Save registers that will be destroyed
	push	bc	
	
	call	ASCII_LOOKUP	; Get beginning of character bitmap in HL	
	
	ld	b,$06		; bytes in pattern
	
CLOOP:	ld	a,(hl)		; Get first byte of bitmap
	ld	(de),a		; expand the lower nybble of the pattern
	inc	de		; 
	ld	(de),a		; and expand the upper nybble of the pattern

	inc	hl		; Next byte of pattern
	inc	de		

	ld	a,(hl)		; Get first byte of bitmap
	ld	(de),a		; expand the lower nybble of the pattern
	inc	de		; 
	ld	(de),a		; and expand the upper nybble of the pattern

	inc	hl		; Set up for next byte of pattern

	push	hl		; add in offset
	ex	de,hl
	ld	de,77
	add	hl,de
	ex	de,hl
	pop	hl

	djnz	CLOOP		; If not end of pattern, go back for more

	ex	de,hl		; add in one more offset for a line between characters
	ld	de,80
	add	hl,de
	ex	de,hl

	pop	bc		; Restore destroyed registers
	pop	hl

	ret			; So it is written, so it has been done...
;
;********************************************************************
; Name:			ASCII_LOOKUP
;
; Title:		ASCII to GRAPHICS Table lookup
;
; Function:		Take an ASCII character and translate it into
;			a graphic entry in a character table. 
; 
; Entry:		A  = ASCII character
;			
; Exit:			HL = Address entry in table which corrosponds 
;			     with the ASCII character.
;
; Registers used:	??? 
;
;********************************************************************
;
ASCII_LOOKUP:

	push	af		; Save registers
	push	bc
	push	de

	ld	hl,SPCE		; Set to space character as default
	
	cp	'Z'+2		; Is it above range of characters?
	jp	p,AL_EXIT	; ... and return if out of range

	sub	' '		; Adjust for lookup 
	jp	m,AL_EXIT	; ... but return if out of range 
				;     (below a space character)

	ld	de,GCHARS	; Point to base of table
	ld	l,a		; Extend index in A to 16 bits
	ld	h,0		;
;
; Each entry is 12 bytes, so multiply x12 to adjust index
;
	add	hl,hl		; x2
	add	hl,hl		; x4
	ld	c,l		; save the A*4 result
	ld	b,h
	add	hl,hl		; x8
	add	hl,bc		; x12

	add	hl,de		; add in base of table
				; HL now holds address of bitmap character
AL_EXIT:
	pop	de
	pop	bc
	pop	af
		
	ret			; Thou are done with thy translation!
;
;******************************************************************************
; Delay routine (~.25 second) NOTE: NEED TO ADJUST FOR WATCHDOG CODE
;******************************************************************************
;

CPMS		EQU	1790		; 1.78977 MHz clock

DELAY:		
		push	bc	
		call	DLY	

LDLP:		
		jp	LDLY1
LDLY1:		jp	LDLY2
LDLY2:		jp	LDLY3
LDLY3:		add	a,0
		djnz	LDLP

		push	af
		in	a,($10)		;Watchdog!
		pop	af

		pop	bc

		ld	a,(DELAY)	
		ret

DLY:		dec	a
		ret	z
		ld	b,+(CPMS/50)-1

		
DLP:		
		jp	DLY1
DLY1:		jp	DLY2
DLY2:		jp	DLY3
DLY3:		add	a,0
		djnz	DLP

		jp	DLY4
DLY4:		jp	DLY5
DLY5:		nop
		jp	DLY


;
;********************************************************************
; Name:			CLRSCRN
;
; Title:		Clear Screen
;
; Origin:		David E. Turner cmdr-dave@spamcop.net
;
; Function:		Clear visible video memory only
; 
; Entry:		No parameters on entry.
;		
; Exit:			Visable video RAM is cleared (set to 00)
;
; Registers used:	AF, BC, DE, HL. 
;
; Other routines used	MFILL
;			Sets up parameters for a memory fill
;			and calls MFILL routine.
;
; Notes:		May be modified later to use a different method
;			to clear the screen... perhaps a pattern board function
;********************************************************************
;

CLRSCRN:	push	hl		; Save registers that will be whacked
		push	bc
		push	af

		ld	hl,VIDMEM	; Set up for block move...
		ld	bc,VMSIZEV	; Viewable area only
		ld	a,$00		; Code to blank memory location
		call	MFILL

		pop	af		; Restore whacked registers
		pop	bc
		pop	hl

		ret

;
;********************************************************************
; Name:			MFILL
;
; Title:		Memory Fill 
;
; Origin:		"Z80 Assembly Language Subroutines" 
;			by Lance A. Leventhal & Winthrop Saville
;			Copyright 1983
;			ISBN:0-07-931091-5
;
; Function:		Fill an area of memory with a value.
; 
; Entry:		HL = Base address
;			BC = Area size
;			A  = Value to be placed in memory
;		
;			Note: A size of 0 is interpreted as 65536
;
; Exit:			Area filled with value
;
; Registers used:	AF, BC, DE, HL. 
;
; Time:			Aproximately 21 cycles per byte plus
;			50 cycles overhead
;			
; Size:			Program	11 bytes
;			Data:	None
;********************************************************************
;

MFILL:		
		ld	(hl),a		;Fill first byte with value
		ld	d,h		;Destination PTR = Source PTR+1
		ld	e,l		;
		inc	de		;
		dec	bc		;Eliminate first byte from count
		ld	a,b		;Are there more bytes to fill?
		or	c
		ret	z		;No, return. Size was 1
		ldir			;Yes, use block move to fill rest
					; by moving value ahead 1 byte
		ret

;
;********************************************************************
; Name:			MCOMP
;
; Title:		Memory Compare
;
; Origin:		N/A
;
; Function:		Compare an area of memory with a value.
; 
; Entry:		HL = Base address
;			BC = Area size
;			A  = Value to be compared
;		
;			Note: A size of 0 is interpreted as 65536
;
; Exit:			HL = Address where memory comparison fails
;			A  = failing value if comparison not equal, 
;			     Entry value if all values equal.
;			F =  Z if all correct
;			     NZ if comparison failed	
;
; Registers used:	AF, BC, DE, HL. 
;
;********************************************************************
;

MCOMP:
		cpi			; A-(HL), HL=HL+1, BC=BC-1
					; Flags: If A=(HL) then Z flag set
					;	 otherwise NZ is set.
					;	 If BC=0, then PO is set
					;	 otherwise, PE is set
		jp	nz,MCOMPE	; Bad compare, jump to error routine
		jp	pe,MCOMP	; Loop if BC <> 0
		ret			; Return if all memory is good

MCOMPE:		dec	hl		; Get failing memory location back
		ld	a,(hl)		; Get failing bit pattern

		ret

;
;**********************************************************************
;
; Begin Data Areas
;
;**********************************************************************
;
;**********************************************************************
;
; Color table (pallette)
;
;**********************************************************************
COLTAB:		db	$51, $7c, $f3, $c7, $00, $56, $09, $9e 	

;
;***********************************************************************
; This is the GORF character set in graphical format.
;
; Since the Gorf monitor is vertical, the character set is lying on it's side.
; To see the characters, convert the bytes into a bitmap two by 6 bytes.
; Example:
; CHAR_B is:	$3F, $F0	--XXXXXXXXXX---- 
;		$3F, $F0	--XXXXXXXXXX----
;		$33, $30	--XX--XX--XX----
;		$33, $30	--XX--XX--XX----
;		$3F, $F0	--XXXXXXXXXX----
;		$1E, $E0	---XXXX-XXX-----
;
; The proportion of the drawing is a little off, but you get the idea...
;
; Also, the character set has to be drawn with the Magic Expand mode. This
; converts each bit into a two bit pixel. The pixel color it is converted
; to is based on the ??? register, which tells what to expand each set bit
; or reset bit into. Read more on this subject in the header file.
;

GCHARS:		; This label points to base of the table for indexing purposes

SPCE:	db	$00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
;
;	The below filler is for future ASCII expansion
;
	db	$00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00	; !
	db	$00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00	; "
	db	$00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00	; #
	db	$00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00	; $
	db	$00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00	; %
	db	$00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00	; &
	db	$00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00	; '
	db	$00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00	; (
	db	$00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00	; )
	db	$00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00	; *
	db	$00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00	; +
	db	$00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00	; ,
	db	$03, $00, $03, $00, $03, $00, $03, $00, $03, $00, $03, $00	; -
	db	$00, $00, $00, $00, $60, $00, $60, $00, $00, $00, $00, $00	; .
	db	$00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00	; /
;	
CHAR_0:	db	$1F, $E0, $3F, $F0, $30, $30, $30, $30, $3F, $F0, $1F, $E0
CHAR_1:	db	$00, $00, $30, $20, $3F, $E0, $3F, $F0, $30, $00, $00, $00
CHAR_2:	db	$3E, $60, $3F, $70, $33, $30, $33, $30, $33, $F0, $31, $E0
CHAR_3:	db	$18, $60, $38, $70, $33, $30, $33, $30, $3F, $F0, $1E, $E0
CHAR_4:	db	$03, $F0, $03, $F0, $03, $00, $03, $00, $3F, $F0, $3F, $F0
CHAR_5:	db	$1B, $F0, $3B, $F0, $33, $30, $33, $30, $3F, $30, $1E, $00
CHAR_6:	db	$1F, $E0, $3F, $F0, $33, $30, $33, $30, $3F, $30, $1E, $00
CHAR_7:	db	$00, $30, $38, $30, $3E, $30, $0F, $B0, $03, $F0, $00, $F0
CHAR_8:	db	$1E, $E0, $3F, $F0, $33, $30, $33, $30, $3F, $F0, $1E, $E0
CHAR_9:	db	$01, $E0, $03, $F0, $33, $30, $33, $30, $3F, $F0, $1F, $E0
COLON:	db	$00, $00, $18, $60, $18, $60, $00, $00, $00, $00, $00, $00
;
SCOLON:	db	$00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00	; ;
	db	$00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00	; <
	db	$00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00	; =
	db	$00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00	; >
	db	$00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00	; ?
	db	$1F, $E0, $20, $10, $21, $90, $22, $50, $22, $50, $13, $E0	; @
;
CHAR_A:	db	$3F, $C0, $3F, $E0, $0C, $70, $0C, $70, $3F, $E0, $3F, $C0
CHAR_B:	db	$3F, $F0, $3F, $F0, $33, $30, $33, $30, $3F, $F0, $1E, $E0
CHAR_C:	db	$1F, $E0, $3F, $F0, $30, $30, $30, $30, $38, $70, $18, $60
CHAR_D:	db	$3F, $F0, $3F, $F0, $30, $30, $30, $30, $3F, $F0, $1F, $E0
CHAR_E:	db	$3F, $F0, $3F, $F0, $33, $30, $33, $30, $33, $30, $30, $30
CHAR_F:	db	$3F, $F0, $3F, $F0, $03, $30, $03, $30, $03, $30, $00, $30
CHAR_G:	db	$1F, $E0, $3F, $F0, $30, $30, $36, $30, $3E, $30, $1E, $20
CHAR_H:	db	$3F, $F0, $3F, $F0, $03, $00, $03, $00, $3F, $F0, $3F, $F0
CHAR_I:	db	$00, $00, $30, $30, $3F, $F0, $3F, $F0, $30, $30, $00, $00
CHAR_J:	db	$1C, $00, $3C, $00, $30, $00, $30, $00, $3F, $F0, $1F, $F0
CHAR_K:	db	$3F, $F0, $3F, $F0, $03, $80, $0F, $C0, $3C, $F0, $38, $70
CHAR_L:	db	$3F, $F0, $3F, $F0, $30, $00, $30, $00, $30, $00, $30, $00
CHAR_M:	db	$3F, $F0, $00, $60, $01, $80, $01, $80, $00, $60, $3F, $F0
CHAR_N:	db	$3F, $F0, $3F, $F0, $01, $C0, $03, $80, $3F, $F0, $3F, $F0
CHAR_O:	db	$1F, $E0, $3F, $F0, $30, $30, $30, $30, $3F, $F0, $1F, $E0
CHAR_P:	db	$3F, $F0, $3F, $F0, $03, $30, $03, $30, $03, $F0, $01, $E0
CHAR_Q:	db	$1F, $E0, $3F, $F0, $20, $30, $28, $30, $1F, $F0, $2F, $E0
CHAR_R:	db	$3F, $F0, $3F, $F0, $03, $30, $07, $30, $3F, $F0, $3D, $E0
CHAR_S:	db	$19, $E0, $3B, $F0, $33, $30, $33, $30, $3F, $70, $1E, $60
CHAR_T:	db	$00, $30, $00, $30, $3F, $F0, $3F, $F0, $00, $30, $00, $30
CHAR_U:	db	$1F, $F0, $3F, $F0, $30, $00, $30, $00, $3F, $F0, $1F, $F0
CHAR_V:	db	$03, $F0, $0F, $F0, $3E, $00, $3E, $00, $0F, $F0, $03, $F0
CHAR_W:	db	$3F, $F0, $0C, $00, $07, $80, $07, $80, $0C, $00, $3F, $F0
CHAR_X:	db	$38, $70, $3C, $F0, $0F, $C0, $0F, $C0, $3C, $F0, $38, $70
CHAR_Y:	db	$00, $70, $00, $F0, $3F, $C0, $3F, $C0, $00, $F0, $00, $70
CHAR_Z:	db	$3C, $30, $3E, $30, $37, $30, $33, $B0, $31, $F0, $30, $F0

;
;******************************************************************************
;				
; Special characters not in normal 7 bit ASCII
;
;******************************************************************************
;

CPYWRT:	db	$1F, $E0, $38, $70, $37, $B0, $37, $B0, $3B, $70, $1F, $E0


;******************************************************************************
;				
; Identification Section - Just text to identify the ROM in a hex/character dump
;
;******************************************************************************
;
	db	"Gorf Memory Diagnostics (GORFMD.BIN)"
	db	"Pre-release version 1.0"
	db	"Copyright 2005 David E. Turner cmdr-dave@spamcop.net"
	db	"Not for resale! Free distribution only!"
	db	"No easter eggs in this version... sorry!"

PEND	equ	$		; End of program code and data

	ds	$1000 - $	; Fill rest of 4K EPROM with NOP's

;	END OF 4K EPROM
